# 変換/無変換キーを0.5秒以上押してるとALTキーとして扱う。
# 0.5秒以内でも変換/無変換キーを押しながら他のキーを押すとALTキーとして扱う。

require 'rkremap'
include Rkremap::KeyCode

rk = Rkremap.new
rk.grab = true

class State
  attr_accessor :converted

  def initialize
    @press = nil
    @converted = false
    @thread = nil
  end

  def press(thread)
    @press = true
    @converted = false
    @thread = thread
  end

  def release
    @press = false
    @converted = false
    @thread.kill
    @thread = nil
  end

  def pressed?
    @press
  end
end

state = {
  KEY_HENKAN => State.new,
  KEY_MUHENKAN => State.new,
}

MAP = {
  KEY_HENKAN => KEY_RIGHTALT,
  KEY_MUHENKAN => KEY_LEFTALT,
}

rk.match(code: [KEY_HENKAN, KEY_MUHENKAN], type: :press) do |event|
  thr = Thread.new(event.code) do |code|
    sleep 0.5
    rk.synchronize do
      if state[code].pressed?
        rk.event(code: MAP[code], type: :press)
        state[code].converted = true
      end
    end
  end
  state[event.code].press(thr)
  event.skip
end

rk.match(code: [KEY_HENKAN, KEY_MUHENKAN], type: :release) do |event|
  if state[event.code].converted
    rk.event(code: MAP[event.code], type: :release)
    event.skip
  else
    rk.event(code: event.code, type: :press)
    rk.event(code: event.code, type: :release)
  end
  state[event.code].release
end

rk.match(code: [KEY_HENKAN, KEY_MUHENKAN], type: :repeat) do |event|
  event.skip
end

rk.match(type: :press) do
  [KEY_HENKAN, KEY_MUHENKAN].each do |code|
    if state[code].pressed?
      rk.event(code: MAP[code], type: :press)
      state[code].converted = true
    end
  end
end

def code2key(code)
  Rkremap::CODE_KEY[code].to_s.sub(/\AKEY_/, '')
end

rk.start do |code, mod|
  key = (mod.select{|_, v| v}.keys + [code]).map{|c| code2key(c)}.join('-')
  puts key
  rk.key(code, mod)
end
