# Mac のショートカット風

require 'rkremap'

include Rkremap::KeyCode

ctrl_map = {
  KEY_A => KEY_HOME,
  KEY_E => KEY_END,
  KEY_B => KEY_LEFT,
  KEY_F => KEY_RIGHT,
  KEY_N => KEY_DOWN,
  KEY_P => KEY_UP,
  KEY_D => KEY_DELETE,
  KEY_H => KEY_BACKSPACE,
  KEY_I => KEY_TAB,
}
mod_disable_ctrl = {
  KEY_LEFTCTRL => false,
  KEY_RIGHTCTRL => false,
}
mod_disable_shift = {
  KEY_LEFTSHIFT => false,
  KEY_RIGHTSHIFT => false,
}
mod_disable_alt = {
  KEY_LEFTALT => false,
  KEY_RIGHTALT => false,
}
mod_disable_meta = {
  KEY_LEFTMETA => false,
  KEY_RIGHTMETA => false,
}
mod_disable_all = {}.merge(mod_disable_ctrl, mod_disable_shift, mod_disable_alt, mod_disable_meta)

rk = Rkremap.new
rk.grab = true
rk.x11 = true
rk.start do |code, mod, app|
  # Emacs や端末等ではそのまま
  if app.class_name == 'Emacs' || app.class_name =~ /terminal/i || app.class_name == 'VirtualBox Machine'
    rk.key(code, mod)
    next
  end

  # ALT+[A-Z]/Enter を Ctrl+[A-Z]/Enter に変換
  if (mod[KEY_LEFTALT] || mod[KEY_RIGHTALT]) && (Rkremap::CODE_KEY[code] =~ /\AKEY_[A-Z]\z/ || code == KEY_ENTER)
    mod.merge!(mod_disable_alt, {KEY_LEFTCTRL => true})
    mod[KEY_LEFTCTRL] = true
    rk.key(code, mod)
    next
  end

  # Ctrl は他のキーに変換
  if mod[KEY_LEFTCTRL] || mod[KEY_RIGHTCTRL]
    # Ctrl+K は行末まで削除
    if code == KEY_K
      rk.key(KEY_END, mod_disable_all.merge({KEY_LEFTSHIFT => true}))
      rk.key(KEY_X, mod_disable_all.merge({KEY_LEFTCTRL => true}))
      next
    end
    # 単純な変換
    if ctrl_map[code]
      rk.key(ctrl_map[code], mod_disable_ctrl)
      next
    end
    # その他の Ctrl+[A-Z] は無視
    if Rkremap::CODE_KEY[code] =~ /\AKEY_[A-Z]\z/
      next
    end
  end

  rk.key(code, mod)
end
