# tmtms 用

require 'rkremap'
include Rkremap::KeyCode

class State
  attr_accessor :converted

  def initialize
    @press = nil
    @converted = false
    @thread = nil
  end

  def press(thread)
    @press = true
    @converted = false
    @thread = thread
  end

  def release
    @press = false
    @converted = false
    @thread.kill
    @thread = nil
  end

  def pressed?
    @press
  end
end

rk = Rkremap.new
rk.grab = true
rk.x11 = true

# 変換/無変換キーを0.5秒以上押してるとALTキーとして扱う。
# 0.5秒以内でも変換/無変換キーを押しながら他のキーを押すとALTキーとして扱う。

state = {
  KEY_HENKAN => State.new,
  KEY_MUHENKAN => State.new,
}

henkan_map = {
  KEY_HENKAN => KEY_RIGHTALT,
  KEY_MUHENKAN => KEY_LEFTALT,
}

rk.match(code: [KEY_HENKAN, KEY_MUHENKAN], type: :press) do |event|
  thr = Thread.new(event.code) do |code|
    sleep 0.5
    rk.synchronize do
      if state[code].pressed?
        rk.event(code: henkan_map[code], type: :press)
        state[code].converted = true
      end
    end
  end
  state[event.code].press(thr)
  event.skip
end

rk.match(code: [KEY_HENKAN, KEY_MUHENKAN], type: :release) do |event|
  if state[event.code].converted
    rk.event(code: henkan_map[event.code], type: :release)
    event.skip
  else
    rk.event(code: event.code, type: :press)
    rk.event(code: event.code, type: :release)
  end
  state[event.code].release
end

rk.match(code: [KEY_HENKAN, KEY_MUHENKAN], type: :repeat) do |event|
  event.skip
end

rk.match(type: :press) do
  [KEY_HENKAN, KEY_MUHENKAN].each do |code|
    if state[code].pressed?
      rk.event(code: henkan_map[code], type: :press)
      state[code].converted = true
    end
  end
end

# CAPSLOCK は CTRL に変換
rk.match(code: KEY_CAPSLOCK) do |event|
  rk.event(code: KEY_LEFTCTRL, type: event.type)
  event.skip
end

ctrl_map = {
  KEY_A => KEY_HOME,
  KEY_E => KEY_END,
  KEY_B => KEY_LEFT,
  KEY_F => KEY_RIGHT,
  KEY_N => KEY_DOWN,
  KEY_P => KEY_UP,
  KEY_D => KEY_DELETE,
  KEY_H => KEY_BACKSPACE,
  KEY_I => KEY_TAB,
}
mod_disable_ctrl = {
  KEY_LEFTCTRL => false,
  KEY_RIGHTCTRL => false,
}
mod_disable_shift = {
  KEY_LEFTSHIFT => false,
  KEY_RIGHTSHIFT => false,
}
mod_disable_alt = {
  KEY_LEFTALT => false,
  KEY_RIGHTALT => false,
}
mod_disable_meta = {
  KEY_LEFTMETA => false,
  KEY_RIGHTMETA => false,
}
mod_disable_all = {}.merge(mod_disable_ctrl, mod_disable_shift, mod_disable_alt, mod_disable_meta)

key_map = {
  KEY_YEN => KEY_BACKSLASH,
  KEY_BACKSLASH => KEY_GRAVE,
  KEY_GRAVE => KEY_ESC,
}

rk.start do |code, mod, app|
  # JISキーボードを英語配列にしたときに一部キーを変える
  code = key_map[code] || code
  if code == KEY_RO
    code = mod[KEY_LEFTSHIFT] || mod[KEY_RIGHTSHIFT] ? KEY_MINUS : KEY_BACKSLASH
  end

  # Emacs や端末等ではそのまま
  if app.class_name == 'Emacs' || app.class_name =~ /terminal/i || app.class_name == 'VirtualBox Machine'
    rk.key(code, mod)
    next
  end

  # ALT+[A-Z]/Enter を Ctrl+[A-Z]/Enter に変換
  if (mod[KEY_LEFTALT] || mod[KEY_RIGHTALT]) && (Rkremap::CODE_KEY[code] =~ /\AKEY_[A-Z]\z/ || code == KEY_ENTER)
    mod.merge!(mod_disable_alt, {KEY_LEFTCTRL => true})
    mod[KEY_LEFTCTRL] = true
    rk.key(code, mod)
    next
  end

  # Ctrl は他のキーに変換
  if mod[KEY_LEFTCTRL] || mod[KEY_RIGHTCTRL]
    # Ctrl+I/O はそのまま
    if code == KEY_I || code ==KEY_O
      rk.key(code, mod)
      next
    end
    # Ctrl+K は行末まで削除
    if code == KEY_K
      rk.key(KEY_END, mod_disable_all.merge({KEY_LEFTSHIFT => true}))
      rk.key(KEY_X, mod_disable_all.merge({KEY_LEFTCTRL => true}))
      next
    end
    # 単純な変換
    if ctrl_map[code]
      rk.key(ctrl_map[code], mod_disable_ctrl)
      next
    end
    # その他の Ctrl+[A-Z] は無視
    if Rkremap::CODE_KEY[code] =~ /\AKEY_[A-Z]\z/
      next
    end
  end

  rk.key(code, mod)
end
