class Rkremap
  EV_KEY = 1
  EV_SYN = 0
  SYN_REPORT = 0

  # https://www.kernel.org/doc/html/v4.12/input/input_uapi.html
  class Evdev
    EVIOCGBIT_ANY = 2147566880     # EVIOCGBIT(0, 1)
    EVIOCGBIT_EV_KEY = 2153792801  # EVIOCGBIT(EV_KEY, (KEY_MAX-1)/8+1)
    EVIOCGRAB = 1074021776
    EVIOCGNAME = 2164278534        # EVIOCGNAME(256)

    @io2evdev = {}
    def self.io2evdev
      @io2evdev
    end

    @device_list = {}

    # @return [Array<Evdev>]
    def self.detect
      devs = []
      new_devs = Dir.children("/dev/input").map do |dev|
        if dev =~ /\Aevent\d+\z/
          path = "/dev/input/#{dev}"
          devs.push path
          next if @device_list[path]
          @device_list[path] = true
          Evdev.new(path)
        end
      end.compact
      (@device_list.keys - devs).each do |path|
        @device_list.delete path
      end
      new_devs
    end

    def self.remove_device(evdev)
      @io2evdev.delete evdev.io
      @device_list.delete evdev.path
    end

    # @param inputs [Array<Evdev>]
    # @param timeout [Numeric]
    # @return [Evdev]
    def self.select(inputs, timeout)
      r, = IO.select(inputs.map(&:io), nil, nil, timeout)
      r && @io2evdev[r[0]]
    end

    # @return [String]
    attr_reader :path

    # @param path [String]
    def initialize(path)
      @path = path
      @io = nil
      @capa = nil
      @uinput = nil
    end

    def eql?(other)
      other.is_a?(Evdev) && self.path == other.path
    end

    # @return [IO]
    def io
      return @io if @io
      @io = File.open(@path)
      self.class.io2evdev[@io] = self
      @io
    end

    def close
      @io&.close
      self.class.remove_device(self)
    end

    # @param key [Integer]
    # @return [Boolean]
    def capable?(key)
      unless @capa
        buf = ' ' * ((KeyCode::KEY_MAX-1)/8+1)
        io.ioctl(EVIOCGBIT_EV_KEY, buf)
        @capa = buf.unpack('C*')
      end
      @capa[key/8][key%8] != 0
    end

    # @return [Boolean]
    def keyboard?
      buf = ' '
      io.ioctl(EVIOCGNAME, buf)
      return false if buf =~ /\Arkremap\0*\z/

      io.ioctl(EVIOCGBIT_ANY, buf)
      return false if buf.unpack1('C')[EV_KEY] == 0
      capable?(KeyCode::KEY_0) && capable?(KeyCode::KEY_9) &&
        capable?(KeyCode::KEY_A) && capable?(KeyCode::KEY_Z) && capable?(KeyCode::KEY_SPACE)
    end

    def grab
      io.ioctl(EVIOCGRAB, 1)
    end

    # struct input_event {
    #   struct timeval time;
    #   unsigned short type;
    #   unsigned short code;
    #   unsigned int value;
    # };
    # @return [Array<Time, Integer, Integer, Integer>]
    def read_event
      raw = io.sysread(24)  # sizeof(struct input_event)
      sec, usec, type, code, value = raw.unpack('Q!Q!SSl')
      time = Time.at(sec, usec)
      return time, type, code, value
    end
  end

  # https://www.kernel.org/doc/html/v4.12/input/uinput.html
  class Uinput
    UI_SET_EVBIT = 1074025828
    UI_SET_KEYBIT = 1074025829
    UI_DEV_SETUP = 1079792899
    UI_DEV_CREATE = 21761

    def initialize
      @dev = File.open('/dev/uinput', 'w')
      setup
    end

    def setup
      @dev.ioctl(UI_SET_EVBIT, EV_KEY)
      KeyCode::KEY_CNT.times{|k| @dev.ioctl(UI_SET_KEYBIT, k)}
      bustype = 0x03    # BUS_USB
      vendor = 0x1234   # てきとー
      product = 0x5678  # てきとー
      version = 1       # てきとー
      name = 'rkremap'
      ff_efects_max = 0
      setup = [bustype, vendor, product, version, name, ff_efects_max].pack("SSSSZ80L") # struct uinput_setup
      @dev.ioctl(UI_DEV_SETUP, setup)
      @dev.ioctl(UI_DEV_CREATE)
    end

    # @param type [Integer] EV_KEY/ EV_SYN
    # @param code [Integer] キーコード / SYN_REPORT
    # @param value [Integer] 0:release / 1:press / 2:repeat
    def write_event(type, code, value)
      @dev.syswrite(['', type, code, value].pack('a16SSl'))
    end
  end
end
